__title__ = "aioauth"
__description__ = "Asynchronous OAuth 2.0 framework for Python 3."
__url__ = "https://github.com/aliev/aioauth"
__version__ = "0.1.1"
__author__ = "Ali Aliyev"
__author_email__ = "ali@aliev.me"
__license__ = "The MIT License (MIT)"
__copyright__ = "Copyright 2020 Ali Aliyev"
